==> externalVector_1.0.3.tar.gz: externalVector/R/init.R <==
library('methods')

## These two functions would be replaced by "attributes"
## and "attributes<-" if and when slots are no longer
## stored as attributes

nonSlotAttributes <-
    function(x)
{
    ans <- attributes(x)
    klass <- class(x)
    if (isClass(klass)) {
        ans[c(slotNames(klass), "class")] <- NULL
    }
    ans
}

"nonSlotAttributes<-" <-
    function(x, value)
{
    if (!(is.null(value) || is.list(value) || is.pairlist(value)))
        stop("attributes must be in a list")
    klass <- class(x)
    if (isClass(klass)) {
        if (length(value) == 0) {
            value <- attributes(x)[c(slotNames(klass), "class")]
        } else {
            nm <- c(slotNames(klass), "class")
            value[nm] <- attributes(x)[nm]
        }
    }
    attributes(x) <- value
    x
}

if (length(findFunction("setVirtualMethod")) == 0) {
    setVirtualMethod <-
        function (name, signature.virtual,
                  signature.nonvirtual = character(),
                  ..., where=topenv(parent.frame()))
    {
        if (length(signature.virtual) == 0)
            stop("virtual method signature must be of non-zero length")
        if (!any(unlist(lapply(signature.virtual, isVirtualClass))))
            stop("virtual method signature can only have VIRTUAL classes")

        f <- getGeneric(name)
        if (is.null(f)) {
            setGeneric(name, ..., where=where)
            f <- getGeneric(name)
        }

        if (length(names(signature.virtual)) !=
            length(signature.virtual) && length(signature.nonvirtual) != 0)
            stop("can not combine virtual and non-virtual parts. ",
                 "Provide names for virtual arguments")
        if (length(names(signature.nonvirtual)) !=
            length(signature.nonvirtual))
            stop("can not combine virtual and non-virtual parts. ",
                 "Provide names for non-virtual arguments")
        signature.method <- 
            c(signature.virtual, signature.nonvirtual)
        sig <- f@signature
        attributes(f) <- NULL
             
        ## Find the virtual parameter names
        mcall <-
            as.list(seq(along=signature.method))
        names(mcall) <- names(signature.method)
        mcall <- do.call("call", c(name, mcall))
        mcall <- match.call(f, mcall)
        mcall <- as.list(mcall[-1])
        nm <- names(mcall)
        notmatched <- !(nm %in% sig)
        if (any(notmatched))
            stop("unused arguments in signature: ", nm[notmatched])
        mcall <- unlist(mcall)
        names(signature.method) <- names(sort(mcall))
        signature.method <- as.list(signature.method)[sig]
        signature.method[unlist(lapply(signature.method,
                                       is.null))] <- NULL
        signature.method <-
            do.call("signature", signature.method)

        ## Find the virtual signature names
        parNames <- names(sort(mcall[mcall <= length(signature.virtual)]))

        ## Create the method definition
        parname <- as.name(paste('.', sort(parNames)[1], sep=''))
        parDef <-
            bquote(function(x) paste(.(parname), ' = "', class(get(.(parname))), '"', sep = ''))
        names(parDef[[2]]) <- as.character(parname)

        body(f) <- bquote({
            stop("Trying to call virtual method for \"", .(name),
                 "\" with signature ",
                 paste(lapply(.(parNames), .(parDef)), collapse = ', '))
        })
        setMethod(name, signature.method, f, where=where)
    }
}

setClass("externalVector", "VIRTUAL")

setClassUnion("vectorNamesType", c("character", "NULL"))
setClassUnion("integerOrNull", c("integer", "NULL"))
setClassUnion("listOrNull", c("list", "NULL"))

if (!isGeneric("any.na")) {
    any.na <- function(x) any(is.na(x))
    setGeneric("any.na")
}

if (!isGeneric("all.na")) {
    all.na <- function(x) all(is.na(x))
    setGeneric("all.na")
}

==> externalVector_1.4.0.tar.gz: externalVector/R/init.R <==
library('methods')

## These two functions would be replaced by "attributes"
## and "attributes<-" if and when slots are no longer
## stored as attributes

nonSlotAttributes <-
    function(x)
{
    ans <- attributes(x)
    klass <- class(x)
    if (isClass(klass)) {
        ans[c(slotNames(klass), "class")] <- NULL
    }
    ans
}

"nonSlotAttributes<-" <-
    function(x, value)
{
    if (!(is.null(value) || is.list(value) || is.pairlist(value)))
        stop("attributes must be in a list")
    klass <- class(x)
    if (isClass(klass)) {
        if (length(value) == 0) {
            value <- attributes(x)[c(slotNames(klass), "class")]
        } else {
            nm <- c(slotNames(klass), "class")
            value[nm] <- attributes(x)[nm]
        }
    }
    attributes(x) <- value
    x
}

if (length(findFunction("setVirtualMethod")) == 0) {
    setVirtualMethod <-
        function (name, signature.virtual,
                  signature.nonvirtual = character(),
                  ..., where=topenv(parent.frame()))
    {
        if (length(signature.virtual) == 0)
            stop("virtual method signature must be of non-zero length")
        if (!any(unlist(lapply(signature.virtual, isVirtualClass))))
            stop("virtual method signature can only have VIRTUAL classes")

        f <- getGeneric(name)
        if (is.null(f)) {
            setGeneric(name, ..., where=where)
            f <- getGeneric(name)
        }

        if (length(names(signature.virtual)) !=
            length(signature.virtual) && length(signature.nonvirtual) != 0)
            stop("can not combine virtual and non-virtual parts. ",
                 "Provide names for virtual arguments")
        if (length(names(signature.nonvirtual)) !=
            length(signature.nonvirtual))
            stop("can not combine virtual and non-virtual parts. ",
                 "Provide names for non-virtual arguments")
        signature.method <-
            c(signature.virtual, signature.nonvirtual)
        sig <- f@signature
        attributes(f) <- NULL

        ## Find the virtual parameter names
        mcall <-
            as.list(seq(along=signature.method))
        names(mcall) <- names(signature.method)
        mcall <- do.call("call", c(name, mcall))
        mcall <- match.call(f, mcall)
        mcall <- as.list(mcall[-1])
        nm <- names(mcall)
        notmatched <- !(nm %in% sig)
        if (any(notmatched))
            stop("unused arguments in signature: ", nm[notmatched])
        mcall <- unlist(mcall)
        names(signature.method) <- names(sort(mcall))
        signature.method <- as.list(signature.method)[sig]
        signature.method[unlist(lapply(signature.method,
                                       is.null))] <- NULL
        signature.method <-
            do.call("signature", signature.method)

        ## Find the virtual signature names
        parNames <- names(sort(mcall[mcall <= length(signature.virtual)]))

        ## Create the method definition
        parname <- as.name(paste('.', sort(parNames)[1], sep=''))
        parDef <-
            bquote(function(x) paste(.(parname), ' = "', class(get(.(parname))), '"', sep = ''))
        names(parDef[[2]]) <- as.character(parname)

        body(f) <- bquote({
            stop("Trying to call virtual method for \"", .(name),
                 "\" with signature ",
                 paste(lapply(.(parNames), .(parDef)), collapse = ', '))
        })
        setMethod(name, signature.method, f, where=where)
    }
}

setClass("externalVector", "VIRTUAL")

setClassUnion("vectorNamesType", c("character", "NULL"))
setClassUnion("integerOrNull", c("integer", "NULL"))
setClassUnion("listOrNull", c("list", "NULL"))

setGeneric("anyNA", function(x) standardGeneric("anyNA"))
setMethod("anyNA", "ANY", function(x) any(is.na(x)))

setGeneric("allNA", function(x) standardGeneric("allNA"))
setMethod("allNA", "ANY", function(x) all(is.na(x)))
